from termcolor import colored
import logging

class Log:

	LEVELS = {
		'debug'   : logging.DEBUG,
		'info'    : logging.INFO,
		'warning' : logging.WARNING,
		'error'   : logging.ERROR,
		'critical': logging.CRITICAL
	}

	CONFIG = {
		'level' : 'debug',
		'format' : '%(message)s',
	}

	def __init__(self, config = {}):
		if config:
			self.config = config
		else:
			self.config = Log.CONFIG

		lvl = Log.LEVELS[self.config['level']] if ('level' in self.config) else Log.LEVELS['info']
		filepath = self.config['file'] if 'file' in self.config else None
		fmt = self.config['format'] if 'format' in self.config else None

		logging.basicConfig(filename = filepath, level = lvl, format = fmt)

	def info(self, msg):
		logging.info(colored(msg, 'green'))

	def warning(self, msg):
		logging.warning(colored(msg, 'yellow', attrs = ['reverse']))

	def debug(self, msg):
		logging.debug(colored(msg, 'cyan', attrs = ['blink']))

	def error(self, msg):
		logging.debug(colored(msg, 'red'))

	def critical(self, msg):
		logging.critical(colored(msg, 'red', attrs = ['reverse']))



# application initiation script
import os, sys, traceback

from glim.facades import Config, Database, Orm, Session, Cookie, IoC, View, Log
from glim.utils import import_module
from glim.dispatch import Glim

import glim.paths as paths

from werkzeug.serving import run_simple
from werkzeug.wsgi import SharedDataMiddleware
from termcolor import colored

class App:

    def __init__(self, commandadapter, env = 'development'):

        self.commandadapter = commandadapter

        self.mconfig = import_module('app.config.%s' % env)
        if self.mconfig is None:
            print colored('Configuration for %s not found' % env, 'red')
            exit()

        self.config = self.mconfig.config

        self.register_config()
        self.register_log()
        self.register_database()
        self.register_extensions()
        self.register_ioc()
        self.register_view()
        
        # find out start
        mstart = import_module('app.start')
        self.before = mstart.before

    def register_config(self):
        Config.register(self.config)

    def register_database(self):
        if 'db' in self.config.keys():
            if self.config['db']:
                Database.register(self.config['db'])
                if 'orm' in self.config.keys():
                    if self.config['orm']:
                        Orm.register(Database.engines)

    def register_extensions(self, extensions = []):
        try:
            for extension, config in self.config['extensions'].items():

                # extension module base string
                ext_bstr = 'ext.%s' % (extension)

                # start script
                ext_sstr = '%s.start' % ext_bstr

                ext_startmodule = import_module(ext_sstr, pass_errors = True)
                if ext_startmodule is not None:
                    before = getattr(ext_startmodule, 'before')
                    before(config)

                # register extension commands if exists
                ext_cmdstr = '%s.%s' % (ext_bstr, 'commands')

                ext_cmd_module = import_module(ext_cmdstr, pass_errors = True)
                if ext_cmd_module is not None:
                    self.commandadapter.register_extension(ext_cmd_module, extension)

        except Exception, e:
            print traceback.format_exc()
            Log.error(e)

    def register_ioc(self):
        IoC.register()

    def register_view(self):
        if 'views' in self.config:
            View.register(self.config['views'])

    def register_log(self):
        if 'log' in self.config:
            Log.register(self.config['log'])
        else:
            Log.register()

    def start(self, host = '127.0.0.1', port = '8080', env = 'development'):
        try:
            self.before()
            mroutes = import_module('app.routes')
            app = Glim(mroutes.urls, self.config['app'])

            if 'static' in self.config['app']:
                app.wsgi_app = SharedDataMiddleware(app.wsgi_app, {
                    self.config['app']['static']['url'] : self.config['app']['static']['path']
                })

            run_simple(host, int(port), app, use_debugger = self.config['app']['debugger'], use_reloader = self.config['app']['reloader'])

        except Exception, e:
            print traceback.format_exc()
            exit()


import inspect

class CommandAdapter:

    def __init__(self, subparsers):
        self.subparsers = subparsers
        self.commands = []

    def retrieve_commands(self, module):
        commands = []

        for name, obj in inspect.getmembers(module):
            if name != 'Command' and 'Command' in name:
                cobject = getattr(module, name)
                commands.append(cobject)

        return commands

    def register(self, module):
        if module is not None:
            cmds = self.retrieve_commands(module)
            commands = []

            for c in cmds:
                cmd = c(self.subparsers)
                self.commands.append(cmd)

    def register_extension(self, module, extension):
        if module is not None:
            cmds = self.retrieve_commands(module)
            commands = []

            for c in cmds:
                name = '%s:%s' % (extension, c.name)
                cmd = c(self.subparsers, name)
                self.commands.append(cmd)

    def match(self, args):
        command = None
        for c in self.commands:
            if c.name == args.which:
                c.args = args
                command = c
                break
        return command

    def is_glimcommand(self, command):
        return isinstance(command, GlimCommand)

    def dispatch(self, command, app):
        if self.is_glimcommand(command):
            command.run(app)
        else:
            command.run()

class Command:

    name = 'base'
    description = 'base command'

    # parser is the root parser
    def __init__(self, subparsers, name = None):

        if name is not None:
            self.name = name

        self.subparsers = subparsers
        self.parser = self.subparsers.add_parser(
            self.name,
            help = self.description
        )
        self.parser.set_defaults(which = self.name)

        self.args = None
        self.configure()

    def configure(self):
        pass

    def add_argument(self, *args, **kwargs):
        self.parser.add_argument(*args, **kwargs)

    def run(self):
        return

class GlimCommand(Command):
    def run(self, app):
        pass


from jinja2 import Environment, PackageLoader
from werkzeug.wrappers import Response

from glim.core import Registry

# Extension class
class Extension():
    pass

# Base conroller class that extends all the controllers
class Controller:

    def __init__(self, request):
        self.request = request

# Rest controller that
class RestController(Controller):

    def get(self):
        pass

    def post(self):
        pass

    def put(self):
        pass

    def patch(self):
        pass

    def delete(self):
        pass

class Service:
    pass

class View:

    def __init__(self, config):
        self.config = config
        package, folder = self.config['package'].split('.')

        self.env = Environment(
            loader = PackageLoader(package, folder
        ))

    # returns a template object given file
    def get(self, file):
        return self.env.get_template(file + '.html')

    def source(self, file, *args, **kwargs):
        tpl = self.get(file)
        return tpl.render(*args, **kwargs)

    def render(self, file, *args, **kwargs):
        return Response(self.source(file, *args, **kwargs), mimetype = 'text/html')



#!/usr/bin/env python
#          _
#       | (_)          
#   __ _| |_ _ __ ___  
#  / _` | | | '_ ` _ \ 
# | (_| | | | | | | | |
#  \__, |_|_|_| |_| |_|
#   __/ |              
#  |___/               
#
#
# author: Aras Can Akin
# description: A modern python framework for the web
# version: 0.8.4

import paths
paths.configure_sys_path()

from glim.app import App
from glim.utils import import_module
from glim.command import CommandAdapter

import glim.commands

import traceback
import argparse
import os
import sys

description = "glim ~ a modern python framework for the web"

def main():
	# register the global parser
	preparser = argparse.ArgumentParser(description = description, add_help = False)
	preparser.add_argument('--env', '-e', dest = 'env', default = 'development', help = 'choose application environment')

	# parse existing options
	namespace, extra = preparser.parse_known_args()
	env = namespace.env

	parser = argparse.ArgumentParser(parents = [preparser], description = description, add_help = True)
	subparsers = parser.add_subparsers(title = 'commands', help = 'commands')

	# initialize a command adapter with subparsers
	commandadapter = CommandAdapter(subparsers)

	# register glim commands
	commandadapter.register(glim.commands)

	# register app commands
	appcommands = import_module('app.commands', pass_errors = True)
	commandadapter.register(appcommands)

	# check if a new app is being created
	new = True if 'new' in extra else False
	if new:
		app = None
	else:
		app = App(commandadapter, env)


	args = parser.parse_args()

	command = commandadapter.match(args)
	commandadapter.dispatch(command, app)

if __name__ == '__main__':
	main()



import os
from termcolor import colored

PROJECT_PATH = os.getcwd()
APP_PATH = os.path.join(PROJECT_PATH, 'app')
GLIM_ROOT_PATH = os.path.dirname(os.path.dirname(__file__))
PROTO_PATH = os.path.join(os.path.dirname(__file__), 'prototype')

EXT_PATH = os.path.join(PROJECT_PATH, 'ext')
VIEWS_PATH = os.path.join(APP_PATH, 'views')
STATIC_PATH = os.path.join(APP_PATH, 'static')
STORAGE_PATH = os.path.join(APP_PATH, 'storage')

import sys
from pprint import pprint as p

def configure_sys_path():
	if GLIM_ROOT_PATH == PROJECT_PATH:
		print colored('Development mode is on, sys.path is being configured', 'yellow')
		sys.path.pop(0)
		sys.path.insert(0, GLIM_ROOT_PATH)
	else:
		sys.path.insert(0, PROJECT_PATH)

def controllers():
	return os.path.join(APP_PATH, 'controllers.py')

def config(env):
	return os.path.join(APP_PATH, 'config', '%s.py' % env)

def start():
	return os.path.join(APP_PATH, 'start.py')

def commands():
	return os.path.join(APP_PATH, 'commands.py')

def routes():
	return os.path.join(APP_PATH, 'routes.py')

def extensions(ext):
	return os.path.join(EXT_PATH, '%s' % ext, '%s.py' % ext)

def extension_commands(ext):
	return os.path.join(EXT_PATH, '%s' % ext, 'commands.py')


from glim.component import Controller
from glim.facades import View
import json

class BaseController(Controller):
	def hello(self):
		return json.dumps({
			'error' : None,
			'msg' : 'Welcome to glim!'
		})



from werkzeug.wrappers import Response as response

# registry class is for generic framework components register function with configuration
class Registry:

    def __init__(self, registrar):
        self.registrar = registrar

    def get(self, key):
        try :
            layers = key.split('.')
            value = self.registrar
            for key in layers:
                value = value[key]
            return value
        except:
            return None

    def set(self, key, value):
        target = self.registrar
        for element in key.split('.')[:-1]:
            target = target.setdefault(element, dict())
        target[key.split(".")[-1]] = value

    def flush(self):
        self.registrar.clear()

    def update(self, registrar):
        self.registrar.update(registrar)

    def all(self):
        return self.registrar

    def __str__(self):
        return self.registrar

# Config class
class Config(Registry):
    pass

# Session class
class Session(Registry):
    pass

# Cookie Class
class Cookie(Registry):
    pass

Response = response

class IoC:

    def __init__(self, instances = {}):
        self.instances = instances

    def bind(self, key, value):
        self.instances[key] = value

    def resolve(self, key):
        try:
            return self.instances[key]
        except:
            return None

# metaclass for facade class
class DeflectToInstance(type):

    # selfcls in order to make clear it is a class object (as we are a metaclass)
    def __getattr__(selfcls, a):
        try:
            # first, inquiry the class itself
            return super(DeflectToInstance, selfcls).__getattr__(a)
        except AttributeError:
            # Not found, so try to inquiry the instance attribute:
            return getattr(selfcls.instance, a)

# facade that is used to hold instances statically with boot method
class Facade:

    __metaclass__ = DeflectToInstance

    instance = None

    # accessor is the object which will be registered during runtime
    accessor = None

    @classmethod
    def boot(cls, *args, **kwargs):
        if cls.accessor is not None:
            if cls.instance is None:
                cls.instance = cls.accessor(*args, **kwargs)

    @classmethod
    def register(cls, config = {}):
        if cls.accessor is not None:
            if cls.instance is None:
                cls.instance = cls.accessor(config)

    @classmethod
    def _get(cls):
        return cls.instance


from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker

from glim.core import Registry

class Database:

    def __init__(self, config):
        self.active = 'default'
        self.config = config
        self.connections = {}
        self.sessions = {}
        self.engines = {}

        for k, config in self.config.items():

            cstring = '%s://%s@%s/%s' % (
                config['driver'],
                config['user'],
                config['password'],
                config['schema']
            )

            engine = create_engine(cstring)
            connection = engine.connect()
            Session = sessionmaker(bind=engine)
            session = Session()

            self.engines[k] = engine
            self.sessions[k] = session
            self.connections[k] = connection

    def __getattr__(self, attr):
        return getattr(self.connections[self.active], attr)

    def session(self):
        return self.sessions[self.active]

    def engine(self, key = None):

        if key:
            return self.engines[k]
        else:
            return self.engines[self.active]

    def connection(self, key = None):
        if key:
            self.active = key
        else:
            self.active = 'default'

        return self

    def get(self, key = None):
        if key:
            return self.connections[key]
        else:
            return self.connections[self.active]

    def close(self, key = None):
        for connection in self.config.items():
            connection.close()

Model = declarative_base()

class Orm:

    def __init__(self, engines):
        self.active = 'default'
        self.engines = engines
        self.sessions = {}
        DBSession = sessionmaker()
        for k, engine in engines.items():
            DBSession.configure(bind = engine)
            self.sessions[k] = DBSession()

    def __getattr__(self, attr):
        return getattr(self.sessions[self.active], attr)

    def session(key = 'default'):
        self.active = key
        return self



from glim.core import Facade, Config as config, IoC as ioc
from glim.db import Database as database, Orm as orm
from glim.component import View as view
from glim.log import Log as log

class Config(Facade):
	accessor = config

class Session(Facade):
	pass

class Cookie(Facade):
	pass

class Database(Facade):
	accessor = database

class Orm(Facade):
	accessor = orm

class IoC(Facade):
	accessor = ioc

class View(Facade):
	accessor = view

class Log(Facade):
	accessor = log


import shutil
import os
import traceback
import imp

# function performs an import given full path & module
def import_source(module, path, pass_errors = False):
    try:
        m = imp.load_source(module, path)
        return m
    except Exception, e:
        return None

# function performs a parametric import statement, returns None if not found
def import_module(module, pass_errors = False):

    frm = module.split('.')
    try:
        m = __import__(module, fromlist = [frm[1]])
        return m
    except ImportError, e:
        if pass_errors:
            return None
        else:
            print traceback.format_exc()
            return None
    except Exception, e:
        print traceback.format_exc()
        return None

# function performs a recursive copy of files and folders in the filesystem
def copytree(src, dst, symlinks = False, ignore = None):
    for item in os.listdir(src):
        s = os.path.join(src, item)
        d = os.path.join(dst, item)
        if os.path.isdir(s):
            shutil.copytree(s, d, symlinks, ignore)
        else:
            shutil.copy2(s, d)


from setuptools import setup
import glim
import os
from os.path import exists

def read(*paths):
    """Build a file path from *paths* and return the contents."""
    with open(os.path.join(*paths), 'r') as f:
        return f.read()

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

setup(
  author = 'Aras Can Akin',
  author_email = 'aacanakin@gmail.com',
  name = 'glim',
  packages = find_packages(exclude=['app', 'ext']),
  version = '0.8.4',
  description = 'A modern framework for the web',
  long_description = read('README.rst'),
  entry_points={
    'console_scripts':['glim=glim.cli:main']
  },
  url = 'https://github.com/aacanakin/glim',
  download_url = 'https://github.com/aacanakin/glim/archive/v0.8.4.zip',
  keywords = [
    'mvc',
    'mvc(s)',
    'framework',
    'web framework',
    'api development',
    'Werkzeug',
    'SQLAlchemy',
    'Jinja2',
    'termcolor',
  ],
  install_requires = [
    "Werkzeug >= 0.9",
    "Jinja2 >= 2.7.3",
    "SQLAlchemy >= 0.9.7",
    "MySQL-python >= 1.2.5",
    "termcolor >= 1.1.0"
  ],
  classifiers = [
    'Development Status :: 3 - Alpha',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules'
  ]
)



import os
from termcolor import colored

from glim.command import GlimCommand
from glim.utils import copytree
from glim.facades import Log

import glim.paths as paths

class NewCommand(GlimCommand):

	name = 'new'
	description = 'generates a new glim app'

	def run(self, app):
		project_path = os.getcwd()
		proto_path = paths.PROTO_PATH

		try:
			copytree(proto_path, project_path)
			print colored('A new glim app created successfully!', 'green')
		except Exception, e:
			print e
			print colored('App already exists', 'red')

class StartCommand(GlimCommand):

	name = 'start'
	description = 'start the glim app web server'

	def configure(self):
		self.add_argument("--host", help = "enter host", default = '127.0.0.1')
		self.add_argument("--port", help = "enter port", default = '8080')

	def run(self, app):
		Log.info('Glim server started on %s environment' % self.args.env)
		app.start(host = self.args.host, port = self.args.port)


from werkzeug.wrappers import Request, Response
from werkzeug.routing import Map, Rule
from werkzeug.exceptions import HTTPException, NotFound
from werkzeug.utils import redirect
from werkzeug.contrib.sessions import FilesystemSessionStore

from glim.utils import import_module
import glim.paths as paths

class Glim:

    def __init__(self, urls = {}, config = {}):
        self.config = config

        try:
            self.session_store = FilesystemSessionStore(self.config['sessions']['path'])
        except:
            self.session_store = None

        ruleset = self.flatten_urls(urls)
        rule_map = []
        for url,rule in ruleset.items():
            rule_map.append(Rule(url, endpoint = rule))

        self.url_map = Map(rule_map)

    def flatten_urls(self, urls, current_key = "", ruleset = {}):
        for key in urls:
            # If the value is of type `dict`, then recurse with the
            # value
            if isinstance(urls[key], dict):
                self.flatten_urls(urls[key], current_key + key)
            # Else if the value is type of list, meaning it is a filter
            elif isinstance(urls[key], (list, tuple)):
                k = ','.join(urls[key])
                ruleset[current_key + key] = k
            else:
                ruleset[current_key + key] = urls[key]

        return ruleset

    def dispatch_request(self, request):
        adapter = self.url_map.bind_to_environ(request.environ)

        try:

            endpoint, values = adapter.match()
            mcontroller = import_module('app.controllers')
            
            # detect filters
            filters = endpoint.split(',')
            endpoint_pieces = filters[-1].split('.')

            # if there exists any filter defined
            if len(filters) > 1:

                filters = filters[:-1]
                # here run filters
                for f in filters:

                    fpieces = f.split('.')
                    cls = fpieces[0]
                    fnc = fpieces[1]
                    mfilter = mcontroller
                    obj = getattr(mfilter, cls)
                    ifilter = obj(request)
                    raw = getattr(ifilter, fnc)(** values)

                    if isinstance(raw, basestring):
                        return Response(raw)

                    if isinstance(raw, Response):
                        return raw

            cls = endpoint_pieces[0]

            restful = False
            try:
                fnc = endpoint_pieces[1]
            except:
                restful = True
                fnc = None

            obj = getattr(mcontroller, cls)
            instance = obj(request)

            raw = None
            if restful:
                raw = getattr(instance, request.method.lower()(**values))
            else:
                raw = getattr(instance, fnc)(** values)

            if isinstance(raw, Response):
                return raw
            else:
                return Response(raw)

        except HTTPException, e:
            return e

    def wsgi_app(self, environ, start_response):

        request = Request(environ)

        if self.session_store is not None:

            sid = request.cookies.get(self.config['sessions']['id_header'])

            if sid is None:
                request.session = self.session_store.new()
            else:
                request.session = self.session_store.get(sid)

        response = self.dispatch_request(request)

        if self.session_store is not None:
            if request.session.should_save:
                self.session_store.save(request.session)
                response.set_cookie(
                    self.config['sessions']['id_header'],
                    request.session.sid
                )

        return response(environ, start_response)

    def __call__(self, environ, start_response):

        return self.wsgi_app(environ, start_response)




import os
import glim.paths

config = {

	'extensions' : {
		# 'gredis' : {
		# 	'default' : {
		# 		'host' : 'localhost',
		# 		'port' : '6379',
		# 		'db'   : 0
		# 	}
		# }
	},

	# database configuration
	'db' : {
		# 'default' : {
		# 	'driver' : 'mysql',
		# 	'host' : 'localhost',
		# 	'schema' : 'test',
		# 	'user' : 'root',
		# 	'password' : '',
		# },
	},

	'orm' : True,
	
	'log' : {
		# 'level' : 'info',
		# 'format' : '[%(levelname)s] : %(message)s',
		# 'file' : 'app/storage/logs/debug.log'
	},

	'views' : {
		# package to be loaded by jinja2
		'package' : 'app.views'
	},

	'sessions' : {
		# session id prefix
		'id_header' : 'glim_session',
		'path' : glim.paths.STORAGE_PATH,
	},
	
	'app' : {
		'reloader' : True,
		'debugger' : True,
		'static' : {
			'path' : glim.paths.STATIC_PATH,
			'url'  : '/static'
		}
	}
}
